//!	Arduino_AnalogInput.h


//--------------------------- inclusion guards ----------------------
#ifndef _Arduino_AnalogInput_h_
#define _Arduino_AnalogInput_h_


//--------------------------- include section -----------------------
#include "EOS_AnalogInput.h"
#include "Arduino.h"


//--------------------------- class definition ----------------------
//! this class represents an abstraction of an analog input used
//! with an Arduino board
//! \code
//! class CPinA0 :
//!         public CArduino_DigitalInput<14>,
//!         public CArduino_DigitalOutput<14>,
//!         public CArduino_AnalogInput<0,1023>
//! \endcode
template <int PIN, int RESOLUTION>
class CArduino_AnalogInput :
        public CEOS_AnalogInput
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CArduino_AnalogInput);


//--------------------------- public types --------------------------
public:


//--------------------------- public access -------------------------
public:
    // toAnalogInput ------------------------------------------------
    //! uses the Arduino function pinMode(pin,mode)
    inline virtual void toAnalogInput()
    {
        pinMode(PIN,INPUT);
    }

    // getAnalogInputResolution -------------------------------------
    inline virtual unsigned int getAnalogInputResolution() const
    {
        return RESOLUTION;
    }

    // readAnalog ---------------------------------------------------
    //! uses the Arduino function analogRead(pin)
    inline virtual unsigned int readAnalog()
    {
        return analogRead(PIN);
    }
};


#endif
