//!	EOS_RealtimeScheduler.cpp


//--------------------------- include section -----------------------
#include "EOS_RealtimeScheduler.h"


//--------------------------- global data ---------------------------
    CEOS_RealtimeScheduler EOS_RealtimeScheduler;


//-------------------------------------------------------------------
// addDelayedTask
void CEOS_RealtimeScheduler::addDelayedTask(CEOS_DelayedTask* pTask,
                                            uint8 nPriority)
{
    SDelayedAndPeriodicTask sTask;
    sTask.pTask = pTask;
    sTask.nPriority = nPriority;

    // Sort in the item
    CEOS_List<SDelayedAndPeriodicTask>::TPosition pPosition = m_DelayedAndPeriodicTaskList.first();
    while (pPosition != NULL)
    {
        const SDelayedAndPeriodicTask& sCheckTask = m_DelayedAndPeriodicTaskList.get(pPosition);
        if (sCheckTask.nPriority < nPriority)
        {
            m_DelayedAndPeriodicTaskList.insertBefore(pPosition,sTask);
            return;
        }
        pPosition = m_DelayedAndPeriodicTaskList.next(pPosition);
    }
    m_DelayedAndPeriodicTaskList.add(sTask);
}


//-------------------------------------------------------------------
// addPeriodicTask
void CEOS_RealtimeScheduler::addPeriodicTask(CEOS_PeriodicTask* pTask,
                                             uint8 nPriority)
{
    // Same list DelayedAndPeriodicTask
    addDelayedTask(pTask,
                   nPriority);
}


//-------------------------------------------------------------------
// schedule
void CEOS_RealtimeScheduler::schedule()
{
    // Realtime scheduler
    m_Mutex.lock();

    // Remember the last value of the realtime scheduler state
    bool bInRTScheduler = m_bInRTScheduler;
    // Now we are definitely in
    m_bInRTScheduler = true;

    // Count down all of the tasks
    CEOS_List<SDelayedAndPeriodicTask>::TPosition pPosition = m_DelayedAndPeriodicTaskList.first();
    while (pPosition != NULL)
    {
        const SDelayedAndPeriodicTask& sTask = m_DelayedAndPeriodicTaskList.get(pPosition);
        if (bInRTScheduler)
        {
            if (m_nPriority >= sTask.nPriority)
            {
                // Does not make sense to continue further, we will not
                // find a better priority
                break;
            }
        }
        // Change the priority, but remember it on the stack
        uint8 nPriority = m_nPriority;
        m_nPriority = sTask.nPriority;
        // Count down the task's internal timer
        sTask.pTask->elapse(1);
        if (sTask.pTask->getTimeout() == 0)
        {
            // Execute the task (without interrupt locked
            m_Mutex.unlock();
            // Execute the realtime function
            sTask.pTask->timeout();
            // block it again, to go on further without disturbance
            m_Mutex.lock();
        }
        if (sTask.pTask->isFinished())
        {
            delete sTask.pTask;
            CEOS_List<SDelayedAndPeriodicTask>::TPosition pNextPosition = m_DelayedAndPeriodicTaskList.next(pPosition);
            m_DelayedAndPeriodicTaskList.remove(pPosition);
            pPosition = pNextPosition;
        }
        else
        {
            pPosition = m_DelayedAndPeriodicTaskList.next(pPosition);
        }
        // Change back the priority
        m_nPriority = nPriority;
    }

    // If we are the first, we are the only who is allowed to reset
    // the state
    if (!bInRTScheduler)
    {
        m_bInRTScheduler = false;
    }
    // Don't forget to leave the critical section
    m_Mutex.unlock();
}



