//!	EOS_DigitalInput.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_DigitalInput_h_
#define _EOS_DigitalInput_h_


//--------------------------- include section -----------------------
#include "EOS.h"


//--------------------------- class definition ----------------------
//! this class represents an abstraction of a digital input
//! \code
//! class CPin :
//!     public CEOS_DigitalInput
//! \endcode
class CEOS_DigitalInput
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_DigitalInput);


//--------------------------- public static access ------------------
public:
    // operator bool_t ----------------------------------------------
    //! \return boolean value of the input
    inline operator bool_t ()
    {
        return readDigital();
    }

    // operator ! ---------------------------------------------------
    //! \return inverted boolean value of the input
    inline bool_t operator ! ()
    {
        return !readDigital();
    }


//--------------------------- public virtual access -----------------
public:
    // toDigitalInput -----------------------------------------------
    //! switches the data direction registers to be an digital input\n
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    virtual void toDigitalInput() = 0;

    // readInput ----------------------------------------------------
    //! \return read value from the port pin\n
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    virtual bool_t readDigital() = 0;
};


#endif
