//!	EOS_IdleTask.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_IdleTask_h_
#define _EOS_IdleTask_h_


//--------------------------- include section -----------------------
#include "EOS_Task.h"


//--------------------------- class definition ----------------------
//! A CEOS_IdleTask represents a task, which is called whenever the
//! scheduler has no other tasks to execute.
//! This class is usually only used by the EOS_Scheduler
class CEOS_IdleTask :
        public CEOS_Task
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! handing over all information to the base class CEOS_Task
    CEOS_IdleTask(void(*pFunc)()) :
        CEOS_Task(pFunc)
    {
        // Nothing to do
    }

    // constructor --------------------------------------------------
    //! \internal
    //! handing over all information to the base class CEOS_Task
    template <class T_OBJECT>
    CEOS_IdleTask(T_OBJECT* pObject,
                  void(T_OBJECT::* pFunc)()) :
        CEOS_Task(pObject,pFunc)
    {
        // Nothing to do
    }

    NO_COPY_AND_ASSIGNMENT(CEOS_IdleTask);


//--------------------------- public access -------------------------
public:
    // isFinished ---------------------------------------------------
    //! an CEOS_Idle task is never finished
    inline bool isFinished() const
    {
        return false;
    }
};

#endif
