/*-------------------------------------------------------------------
PROJECT:	general usage
COPYRIGHT:	Ehrlich GmbH
FILENAME:	eos_types.h
AUTHOR:		Stefan Ehrlich
SUBSTITUTE: nobody
LANGUAGE:	C/C++
COMPILER:	all
DESCRIPTION:
general types for all projects

Revision 2.0: 03.Jan.2004 Stefan Ehrlich
	layout for coding requirements

Revision 1.1: 18.Mar.2001 Claus Doerler
	added HIGH/LOW

Revision 1.0: 07.Mar.2000 Stefan Ehrlich
    first version
---------------------------------------------------------------------
*/


//--------------------------- inclusion guards ----------------------
#ifndef _types_h_
#define _types_h_

//--------------------------- standard types ------------------------
//--------------------------- standard types ------------------------
// sint8 ------------------------------------------------------------
//! signed integer with 1 byte length
//! used for ordinal counting, computing, etc.
//! range -128..+127
typedef signed char sint8;

// uint8 ------------------------------------------------------------
//! unsigned integer with 1 byte length
//! used for ordinal counting, computing, etc.
//! range 0..+255
typedef unsigned char uint8;

// sint16 ------------------------------------------------------------
//! signed integer with 2 bytes length
//! used for ordinal counting, computing, etc.
//! range -32768..+32767
typedef signed short int sint16;

// uint16 ------------------------------------------------------------
//! unsigned integer with 2 byte length
//! used for ordinal counting, computing, etc.
//! range 0..+65536
typedef unsigned short int uint16;

// sint32 ------------------------------------------------------------
//! signed integer with 4 bytes length
//! used for ordinal counting, computing, etc.
//! range -2147483648..+2147483647
typedef signed long int sint32;

// uint32 ------------------------------------------------------------
//! signed integer with 4 bytes length
//! used for ordinal counting, computing, etc.
//! range 0..+4294967296
typedef unsigned long int uint32;

// bool_t -----------------------------------------------------------
//! boolean type
//! range (should be) 0 and 1
typedef uint8 bool_t;


//--------------------------- additional definitions ----------------
// BYTE -------------------------------------------------------------
//! a BYTE should be used for bit arithmetic and for signalling, that
//! one byte is used
typedef uint8 BYTE;

// WORD -------------------------------------------------------------
//! a WORD should be used for bit arithmetic and for signalling, that
//! one word of 2 bytes is used
typedef uint16 WORD;

// DWORD ------------------------------------------------------------
//! a DWORD should be used for bit arithmetic and for signalling, 
//! that one double word of 4 bytes is used
typedef uint32 DWORD;

// bit_t --------------------------------------------------------------
//! some compilers have a bit-type, so use them if possible
#ifdef asdbit
#define bit_t bit
#else
#define bit_t uint8
#endif





#endif

