//	Queue.h
//  this file handles a queue


//--------------------------- inclusion guards ----------------------
#ifndef _Queue_h_
#define	_Queue_h_


//--------------------------- include section -----------------------
#include "types.h"


//--------------------------- class definition ----------------------
template <class T_DATA, class T_POS, int N_SIZE> class CQueue
{
//--------------------------- construction --------------------------
public:
    // Constructor --------------------------------------------------
    inline CQueue() :
        m_nWritePos(0),
        m_nReadPos(0),
        m_nItems(0)
    {
        // Nothing else to do
    }

    // Destructor ---------------------------------------------------
    inline ~CQueue()
    {
        // Nothing to do
    }

    // lock copy and assignement ------------------------------------
private:
    CQueue(const CQueue& Queue);
    const CQueue& operator = (const CQueue& Queue);


//--------------------------- public access -------------------------
public:
    // addItem ------------------------------------------------------
    bool_t addItem(const T_DATA& vData)
    {
        if (!isFull())
        {
            m_nItems++;
            m_nWritePos++;
            if (m_nWritePos >= N_SIZE)
            {
                m_nWritePos = 0;
            }
            m_DataArray[m_nWritePos] = vData;
            return true;
        }
        return false;
    }

    // accessItem ---------------------------------------------------
    T_DATA& accessItem(T_POS nPos)
    {
        nPos += m_nReadPos + 1;
        while (nPos >= N_SIZE)
        {
            nPos -= N_SIZE;
        }
        return m_DataArray[nPos];
    }

    // removeItem ---------------------------------------------------
    void removeItem()
    {
        m_nReadPos++;
        if (m_nReadPos >= N_SIZE)
        {
            m_nReadPos = 0;
        }
        m_nItems--;
    }

    // getItem ------------------------------------------------------
    inline T_DATA getItem()
    {
        removeItem();
        return m_DataArray[m_nReadPos];
    }

    // clear --------------------------------------------------------
    inline void clear()
    {
        m_nWritePos = 0;
        m_nReadPos = 0;
        m_nItems = 0;
    }

    // items --------------------------------------------------------
    inline T_POS items() const
    {
        return m_nItems;
    }

    // size ---------------------------------------------------------
    inline T_POS size() const
    {
        return N_SIZE;
    }

    // isEmpty ------------------------------------------------------
    inline bool_t isEmpty()
    {
        return (m_nItems == 0);
    }

    // isFull -------------------------------------------------------
    inline bool_t isFull()
    {
        return (m_nItems >= N_SIZE);
    }


//--------------------------- protected data ------------------------
protected:
    // m_DataArray --------------------------------------------------
    T_DATA m_DataArray[N_SIZE];

    // m_nWritePos --------------------------------------------------
    T_POS m_nWritePos;

    // m_nReadPos ---------------------------------------------------
    T_POS m_nReadPos;

    // m_nItems -----------------------------------------------------
    T_POS m_nItems;

};


//--------------------------- derived classes -----------------------
template<class T_DATA,int N_SIZE> class CQueue8 : public CQueue<T_DATA,uint8,N_SIZE>{};
template<class T_DATA,int N_SIZE> class CQueue16 : public CQueue<T_DATA,uint16,N_SIZE>{};
template<class T_DATA,int N_SIZE> class CQueue32 : public CQueue<T_DATA,uint32,N_SIZE>{};



#endif  // Inclusion guard

